#!/bin/bash
#

# buildMPICH.sh debug|prof [tar] [source]

prod_top=${PRODUCTS%%:*}/
prod_top=${prod_top%%/}

make_tarball()
{
    echo ""
    set -x
    cd "${prod_top}"
    tarname="${package}-${pkgdotver}-${OS}-${plat}-${qual}.tar.bz2"
    tar cjf "${tarname}" \
       "${package}/${pkgver}.version/${flvr}_${qual}" \
       "${package}/${pkgver}/ups" \
       "${package}/${pkgver}/${flvr}-${qual}"
    cd - >/dev/null 2>&1
    set +x
    exit 0
}

make_source_tarball()
{
    set -x
    cd "${prod_top}"
    tar cjf "${package}-${pkgdotver}-source.tar.bz2" \
       "${package}/${pkgver}/"*.sh \
       "${package}/${pkgver}/ups" \
       "${package}/${pkgver}/source/${pkgtarfile}"
    cd - >/dev/null 2>&1
    set +x
}

if [[ "${1}" == [Dd]* ]]; then
  qual=debug
elif [[ "${1}" == [Pp]* ]]; then
  qual=prof
else
  echo "ERROR: must specify qual=debug|prof" 1>&2
  exit 1
fi
shift


maketar=${1}
sourcetar=${2}

package=mpich
pkgver=v3_0_2
pkgdotver=3.0.2
pkgtarstem=mpich-3.0.2
pkgtarfile=${pkgtarstem}.tar.gz

thisdir=`/bin/pwd`
(( j_arg = \
`expr \`cat /proc/cpuinfo 2>/dev/null | grep -c -e '^processor'\` + 1` ))

if [ "${sourcetar}" = "source" ]
then
  make_source_tarball
fi

source ${prod_top}/setup

OS1=`uname`
if [ "${OS1}" = "Darwin" ]
then
    flvr=`ups flavor -2`
    plat=`uname -m`
    OSnum=`uname -r | cut -f1 -d"."`
    macos=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1,2`
    macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1`
    macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 2`
    #OS=${OS1}${OSnum}
    OS=mac${macos1}${macos2}
else
    flvr=`ups flavor`
    plat=`uname -p`
    OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
    OS=${OS1}${OSnum}
    if [ `lsb_release -d | cut  -f3 -d" "` = "SLF" ]
    then
       OS=slf${OSnum}
    fi
   if [ `lsb_release -i | cut -f2` = "ScientificFermi" ]
   then
      OS=slf${OSnum}
   fi
    if [ `lsb_release -d | cut  -f4 -d" "` = "SLC" ]
    then
       OS=slc${OSnum}
    fi
    if [ `lsb_release -d | cut  -f4 -d" "` = "LTS" ]
    then
       OS=slf${OSnum}
    fi
fi

if [ -z "${flvr}" ]
then
   echo "ERROR: could not determine flavor for this platform"
   echo "       something is very wrong"
   exit 1
fi
pkgdir="$(cd "${prod_top}/mpich/${pkgver}" && /bin/pwd)/${flvr}-${qual}"

sourcedir=$(cd "${prod_top}/mpich/${pkgver}/source" && /bin/pwd)

# make the build directory
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

cd "${pkgdir}"
tar xf "${sourcedir}/${pkgtarfile}"
cd "${pkgtarstem}"

# Common configure args

# Note: building CXX: caveat emptor.
common_config_args="--prefix=${pkgdir}
 --enable-f77 --enable-fc --enable-cxx --enable-romio --enable-debuginfo
 --enable-smpcoll --enable-versioning --enable-threads=runtime
 --disable-wrapper-rpath --enable-shared --enable-static
 --enable-timer-type=clock_gettime --enable-g=debug"

# Debug configure args
debug_config_args="--enable-error-checking=all --enable-error-messages=all
 --enable-timing=all --enable-mpit-pvars=all
 --disable-fast"

# Profile configure args
prof_config_args=" --enable-mpit-pvars=none --enable-strict
 --enable-fast=nochkmsg,notiming,ndebug,nompit,O3"

set -x

# Configure
./configure ${common_config_args} \
 $(eval echo \$\{${qual}_config_args\})

make -j ${j_arg} all || { echo "Error building! Abort!" 1>&2; exit 1; }

# Need to re-link execs to avoid dynamic dependency on mpl.
#
# Yes, this is absolutely horrible and prone to change with the wind and
# the price of rice. libtool is truly evil.
cd src/pm/hydra
for i in hydra_{nameserver,persist,pmi_proxy} mpiexec.hydra; do
  out=.libs/${i}
  sources=${i%%.*}*.o
  libs=".libs/libhydra.a ../../mpl/.libs/libmpl.a tools/topo/hwloc/hwloc/src/.libs/libhwloc_embedded.a -lpthread -lm"
  case ${out} in
      */mpiexec.hydra)
          sources="${sources} utils.o"
          libs=".libs/libpm.a ${libs}"
          ;;
      *)
          :
  esac
  set -x
  gcc -o ${out} ${sources} ${libs} || { echo "Error relinking ${out}\!" 1>&2; break; }
  set +x
done
cd - 2>&1 >/dev/null

make install || { echo "Error installing!" 1>&2; exit 1; }

set +x

# Clean up.
cd "${pkgdir}"

# Wrapper scripts and config files.
find bin etc -type f | while read file; do
  [[ "$(file "${file}")" == *"text"* ]] && \
   perl -wapi\~ -e 's&\Q'"${pkgdir}"'\E&\${MPICH_FQ_DIR}&g' "${file}"
done

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

# this should not complain
echo "Finished building mpich"
echo "Declaring mpich ${pkgver} for ${flvr} -q ${qual}"
if [ ${OS1} = "Darwin" ]
then
   ups declare mpich ${pkgver} -r mpich/${pkgver} -2 -m mpich.table -q ${qual}
else 
   ups declare mpich ${pkgver} -r mpich/${pkgver} -4 -m mpich.table -q ${qual}
fi
setup mpich ${pkgver} -q ${qual}
echo "mpich is installed at ${MPICH_FQ_DIR}"

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

exit 0
